#
# This file is part of TransportMaps.
#
# TransportMaps is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# TransportMaps is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with TransportMaps.  If not, see <http://www.gnu.org/licenses/>.
#
# Transport Maps Library
# Copyright (C) 2015-2017 Massachusetts Institute of Technology
# Uncertainty Quantification group
# Department of Aeronautics and Astronautics
#
# Authors: Transport Map Team
# Website: transportmaps.mit.edu
# Support: transportmaps.mit.edu/qa/
#

import TransportMaps as TM

__all__ = ['Sampler']

class Sampler(TM.TMO):
    r""" Generic sampler of distribution ``d``

    This main class just mirrors all the sampling methods provided by the distribution ``d``.

    Args:
      d (Distributions.Distribution): distribution to sample from.
    """
    def __init__(self, d):
        super(Sampler, self).__init__()
        self.distribution = d

    def rvs(self, m, *args, **kwargs):
        r""" Generate :math:`m` samples and weights from the distribution

        Args:
          m (int): number of samples to generate

        Returns:
          (:class:`tuple` (:class:`ndarray<numpy.ndarray>` [:math:`m,d`], :class:`ndarray<numpy.ndarray>` [:math:`m`])) -- list of points and weights
        """
        return (self.distribution.rvs(m, *args, **kwargs), np.ones(m)/float(m))
    


#
# This file is part of TransportMaps.
#
# TransportMaps is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# TransportMaps is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with TransportMaps.  If not, see <http://www.gnu.org/licenses/>.
#
# Transport Maps Library
# Copyright (C) 2015-2017 Massachusetts Institute of Technology
# Uncertainty Quantification group
# Department of Aeronautics and Astronautics
#
# Authors: Transport Map Team
# Website: transportmaps.mit.edu
# Support: transportmaps.mit.edu/qa/
#

from TransportMaps.Algorithms.SequentialInference.SequentialInferenceBase import *
from TransportMaps.Algorithms.Adaptivity.KullbackLeiblerAdaptivity import KullbackLeiblerBuilder
from TransportMaps.Algorithms.Adaptivity.RegressionAdaptivity import L2RegressionBuilder
from TransportMaps.Distributions import StandardNormalDistribution, \
    PushForwardTransportMapDistribution, PullBackTransportMapDistribution
from TransportMaps.Maps import TransportMap, TriangularTransportMap, \
    CompositeTransportMap, ListCompositeTransportMap, PermutationTransportMap, \
    TriangularListStackedTransportMap
from TransportMaps.Maps.Decomposable import LiftedTransportMap
from TransportMaps.Diagnostics.Routines import variance_approx_kl
from TransportMaps.Routines import L2_misfit

__all__ = ['TransportMapsSmoother']

class TransportMapsSmoother(Smoother):
    r""" Perform the on-line assimilation of a sequential Hidded Markov chain.

    Given the prior distribution on the hyper-parameters :math:`\pi(\Theta)`,
    provides the functions neccessary to assimilate new pieces of data or
    missing data 
    (defined in terms of transition densities
    :math:`\pi\left({\bf Z}_{k+1} \middle\vert {\bf Z}_k, \Theta \right)`
    and log-likelihoods
    :math:`\log \mathcal{L}\left({\bf y}_{k+1}\middle\vert {\bf Z}_{k+1}, \Theta\right)`),
    to return the map pushing forward :math:`\mathcal{N}(0,{\bf I})`
    to the smoothing distribution
    :math:`\pi\left(\Theta, {\bf Z}_\Lambda \middle\vert {\bf y}_\Xi \right)`
    and to return the maps pushing forward :math:`\mathcal{N}(0,{\bf I})`
    to the filtering/forecast distributions
    :math:`\{\pi\left(\Theta, {\bf Z}_k \middle\vert {\bf y}_{0:k} \right)\}_k`.

    For more details see also :cite:`Spantini2017` and the
    `tutorial <example-sequential-stocvol-6d.html>`_.

    Args:
      pi_hyper (:class:`Distribution<TransportMaps.Distributions.Distribution>`):
        prior distribution on the hyper-parameters :math:`\pi(\Theta)`
    """
    def __init__(self, *args, **kwargs):
        super(TransportMapsSmoother, self).__init__(*args, **kwargs)
        self._var_diag_convergence = []
        self._regression_convergence = []

    @property
    def var_diag_convergence(self):
        return self._var_diag_convergence

    @property
    def regression_convergence(self):
        return self._regression_convergence
        
    def _assimilation_step(self, tm, solve_params, builder=None,
                           hyper_tm=None, regression_params=None,
                           hyper_builder=None,
                           skip_initial=False,
                           var_diag_convergence_params=None,
                           regression_convergence_params=None,
                           continue_on_error=True):
        r""" Assimilate one piece of data :math:`\left( \pi\left({\bf Z}_{k+1} \middle\vert {\bf Z}_k, \Theta \right), \log \mathcal{L}\left({\bf y}_{k+1}\middle\vert {\bf Z}_{k+1}, \Theta\right) \right)`.

        Given the new piece of data
        :math:`\left( \pi\left({\bf Z}_{k+1} \middle\vert {\bf Z}_k, \Theta \right), \log \mathcal{L}\left({\bf y}_{k+1}\middle\vert {\bf Z}_{k+1}, \Theta\right) \right)`,
        retrieve the :math:`k`-th Markov component :math:`\pi^k` of :math:`\pi`,
        determine the transport map

        .. math::

           \mathfrak{M}_k({\boldsymbol \theta}, {\bf z}_k, {\bf z}_{k+1}) = \left[
           \begin{array}{l}
           \mathfrak{M}^\Theta_k({\boldsymbol \theta}) \\
           \mathfrak{M}^0_k({\boldsymbol \theta}, {\bf z}_k, {\bf z}_{k+1}) \\
           \mathfrak{M}^1_k({\boldsymbol \theta}, {\bf z}_{k+1})
           \end{array}
           \right] = Q \circ R_k \circ Q

        that pushes forward :math:`\mathcal{N}(0,{\bf I})` to :math:`\pi^k`, and
        embed it into the linear map which will remove the desired conditional
        dependencies from :math:`\pi`.
        
        Optionally, it will also compress the maps
        :math:`\mathfrak{M}_{0}^\Theta \circ \ldots \circ \mathfrak{M}_{k-1}^\Theta`
        into the map :math:`\mathfrak{H}_{k-1}` in order to speed up the
        evaluation of the :math:`k`-th Markov component :math:`\pi^k`.

        Args:
          tm (:class:`TransportMap<TransportMaps.Maps.TransportMap>`):
            transport map :math:`R_k`
          solve_params (dict): parameters to be passed to
            :func:`minimize_kl_divergence<TransportMaps.Maps.TransportMap.minimize_kl_divergence>`
          hyper_tm (:class:`TransportMap<TransportMaps.Maps.TransportMap>`):
            transport map :math:`\mathfrak{H}_{k-1}`
          regression_params (dict): parameters to be passed to
            :func:`regression<TransportMaps.Maps.TransportMap.regression>` during
            the determination of :math:`\mathfrak{H}_{k-1}`
          skip_initial (bool): whether to skip the approximation of
            :math:`\pi\left(\Theta, {\bf Z}_0 \middle\vert {\bf y}_0 \right)` in the first step.
          var_diag_convergence_params (dict): parameters to be used to monitor the
            convergence of the map approximation. If ``None`` the conevergence is not monitored.
          regression_convergence_params (dict): parameters to be used to monitor the
            convergence of the regression step on the hyper-parameters map.
            If ``None`` the conevergence is not monitored.
          continue_on_error (bool): whether to continue when the KL-minimization step or the
            regression step fails with back-up plans

        Raises:
          RunTimeError: an convergence error occurred during the assimilation

        .. see:: :func:`Smoother.assimilate`
        """
        def terminate_kl(log, continue_on_error):
            if not log['success']:
                if continue_on_error:
                    self.logger.warning(
                        log['msg'] + \
                        "Reverted to last converged map. " + \
                        "This may lead to overall loss of accuracy.")
                else:
                    self.logger.error(
                        log['msg'] + " Terminating."
                    )
                    return True
            return False
        
        if builder is None:
            builder = KullbackLeiblerBuilder()
        if hyper_builder is None:
            hyper_builder = L2RegressionBuilder()
        # Approximation
        if not skip_initial and self.nsteps == 0:
            # If step zero, then just approximate self.pi
            rho = StandardNormalDistribution(self.pi.dim)
            builder.base_distribution = rho
            builder.target_distribution = self.pi
            tm, log = builder.solve(tm, **solve_params)
            if terminate_kl(log, continue_on_error):
                raise RuntimeError("KL-minimization did not converge. Terminating.")
            self.L_list.append( tm )
        elif self.nsteps == 1:
            # If step one, then approximate 0-th Markov component
            hdim = self.pi.hyper_dim
            sdim = self.pi.state_dim
            self._regression_convergence.append( None )
            pi0 = self.pi.get_MarkovComponent(0)
            self.rho_mk = StandardNormalDistribution(pi0.dim)
            self.Q = PermutationTransportMap(
                list(range(hdim)) + \
                list(range(hdim+sdim, hdim+2*sdim)) + \
                list(range(hdim, hdim+sdim)) )
            pull_Q_pi0 = PullBackTransportMapDistribution(self.Q, pi0)
            builder.base_distribution = self.rho_mk
            builder.target_distribution = pull_Q_pi0
            tm, log = builder.solve(tm, **solve_params)
            if terminate_kl(log, continue_on_error):
                raise RuntimeError(log['msg'] + " Terminating.")
            # H_list contains the hyper-parameters maps H
            # R_list contains the lower triangular maps R
            # M_list contains the generalized lower triangular maps M
            # L_list contains the lifted maps
            self.H_list = [
                TriangularTransportMap( tm.active_vars[:hdim],
                                        tm.approx_list[:hdim] )
                if hdim > 0 else None ]
            self.R_list = [ tm ]
            self.M_list = [ ListCompositeTransportMap([self.Q, tm, self.Q]) ]
            self.L_list[0] = LiftedTransportMap(0, self.M_list[0], self.pi.dim, hdim)
        elif self.nsteps > 1:
            # If step k, then approximate the (k-1)-th Markov component
            hdim = self.pi.hyper_dim
            sdim = self.pi.state_dim
            if self.nsteps > 2 and hyper_tm is not None:
                hyper_builder.target_map = self.H_list[-1]
                x0 = None
                if self.nsteps > 3 and isinstance(hyper_builder, L2RegressionBuilder):
                    x0 = self.H_list[-1].t1.coeffs
                hyper_tm, log_list = hyper_builder.solve(
                    hyper_tm, x0=x0, **regression_params)
                if not log_list[-1]['success']:
                    if not continue_on_error:
                        self.logger.error("Regression did not converge. Terminating.")
                        raise RuntimeError("KL-minimization did not converge. Terminating.")
                else:
                    if regression_convergence_params is not None:
                        self._regression_convergence.append(
                            L2_misfit(
                                self.H_list[-1], hyper_tm,
                                **regression_convergence_params) )
                    else:
                        self._regression_convergence.append( None )
                    self.H_list[-1] = hyper_tm
            else:
                self._regression_convergence.append( 0. )
            Mkm1 = TransportMap( self.R_list[-1].active_vars[hdim:hdim+sdim],
                                 self.R_list[-1].approx_list[hdim:hdim+sdim] )
            pik = self.pi.get_MarkovComponent(
                self.nsteps-1, state_map=Mkm1, hyper_map=self.H_list[-1] )
            pull_Q_pik = PullBackTransportMapDistribution(self.Q, pik)
            builder.base_distribution = self.rho_mk
            builder.target_distribution = pull_Q_pik
            tm, log = builder.solve(tm, **solve_params)
            if terminate_kl(log, continue_on_error):
                raise RuntimeError(log['msg'] + " Terminating.")
            self.R_list.append( tm )
            self.M_list.append( ListCompositeTransportMap([self.Q, tm, self.Q]) )
            # Update dimension of all lifted maps
            for L in self.L_list:
                L.dim = L.dim_in = L.dim_out = self.pi.dim
            L = LiftedTransportMap( self.nsteps-1, self.M_list[-1], self.pi.dim, hdim)
            self.L_list.append(L)
            # Store next hyper map composition
            self.H_list.append(
                CompositeTransportMap(
                    self.H_list[-1], TriangularTransportMap(
                        self.R_list[-1].active_vars[:hdim],
                        self.R_list[-1].approx_list[:hdim] ) )
                if hdim > 0 else None )

        # Prepare the filtering maps
        if self.nsteps == 0:
            F = self.L_list[0]
        else:
            hdim = self.pi.hyper_dim
            sdim = self.pi.state_dim
            H = self.H_list[-1]
            R = self.R_list[-1]
            Rkp1 = TriangularTransportMap( R.active_vars[hdim:hdim+sdim],
                                           R.approx_list[hdim:hdim+sdim] )
            if hdim > 0:
                F = TriangularListStackedTransportMap( [H, Rkp1] )
            else:
                F = Rkp1
        self.F_list.append( F )

        # Monitor kl convergence
        if var_diag_convergence_params is not None:
            pull_tar = PullBackTransportMapDistribution(
                tm, builder.target_distribution)
            var = variance_approx_kl(
                builder.base_distribution, pull_tar,
                **var_diag_convergence_params)
        else:
            var = None
                        
        if self.nsteps == 1:
            self._var_diag_convergence[0] = var
        else:
            self._var_diag_convergence.append( var )

    def trim(self, ntrim):
        r""" Trim the integrator to ``ntrim``
        """
        nback = self.nsteps - ntrim
        ns = TransportMapsSmoother(self.pi.pi_hyper)
        # Trim smoother lists
        ns.H_list = self.H_list[:ntrim-1]
        ns.R_list = self.R_list[:ntrim-1]
        ns.M_list = self.M_list[:ntrim-1]
        ns.L_list = self.L_list[:ntrim-1]
        for L in ns.L_list:
            L.dim = L.dim_in = L.dim_out = ntrim + self.pi.hyper_dim
        ns.F_list = self.F_list[:ntrim]
        ns._var_diag_convergence = self.var_diag_convergence[:ntrim-1]
        ns._regression_convergence = self.regression_convergence[:ntrim-1]
        # Trim target distribution
        for pi, ll in zip(self.pi.prior.pi_list[:ntrim], self.pi.ll_list[:ntrim]):
            ns.pi.append(pi, ll)
        # Update nsteps
        ns._nsteps = ntrim
        return ns


#
# This file is part of TransportMaps.
#
# TransportMaps is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# TransportMaps is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with TransportMaps.  If not, see <http://www.gnu.org/licenses/>.
#
# Transport Maps Library
# Copyright (C) 2015-2017 Massachusetts Institute of Technology
# Uncertainty Quantification group
# Department of Aeronautics and Astronautics
#
# Authors: Transport Map Team
# Website: transportmaps.mit.edu
# Support: transportmaps.mit.edu/qa/
#

import sys
import logging
import os.path
import dill

__all__ = ['TMO']

class TMO(object):
    r""" Base object for every object in the module.

    This object provides functions for storage and parallelization.
    """
    def __init__(self):
        self.set_logger()

    def set_logger(self):
        import TransportMaps as TM
        self.logger = logging.getLogger("TM." + self.__class__.__name__)
        self.logger.setLevel(TM.LOG_LEVEL)
        # self.logger = logging.getLogger(self.__module__ + "." + self.__class__.__name__)
        if len(self.logger.handlers) == 0:
            self.logger.propagate = False
            ch = logging.StreamHandler(sys.stdout)
            formatter = logging.Formatter("%(asctime)s %(levelname)s: %(name)s: %(message)s",
                                          "%Y-%m-%d %H:%M:%S")
            ch.setFormatter(formatter)
            self.logger.addHandler(ch)

    def __getstate__(self):
        # Avoid pickling un-pickable objects:
        # logger
        odict = self.__dict__.copy()
        odict.pop('logger', None)
        return odict

    def __setstate__(self, idict):
        # Reset removed fields:
        # logger
        self.set_logger()
        self.__dict__.update(idict)

    def store(self, fname, force=False):
        r""" Store the object with the selected file name ``fname``

        Args:
          fname (str): file name
          force (bool): whether to force overwriting
        """
        if os.path.exists(fname) and not force:
            if sys.version_info[0] == 3:
                sel = input("The file %s already exists. " % fname + \
                            "Do you want to overwrite? [y/N] ")
            else:
                sel = raw_input("The file %s already exists. " % fname + \
                                "Do you want to overwrite? [y/N] ")
            if sel != 'y' and sel != 'Y':
                print("Not storing")
                return
        with open(fname, 'wb') as out_stream:
            dill.dump(self, out_stream)



#
# This file is part of TransportMaps.
#
# TransportMaps is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# TransportMaps is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with TransportMaps.  If not, see <http://www.gnu.org/licenses/>.
#
# Transport Maps Library
# Copyright (C) 2015-2017 Massachusetts Institute of Technology
# Uncertainty Quantification group
# Department of Aeronautics and Astronautics
#
# Author: Transport Map Team
# Website: transportmaps.mit.edu
# Support: transportmaps.mit.edu/qa/
#

import unittest
import numpy.random as npr

try:
    import mpi_map
    MPI_SUPPORT = True
except:
    MPI_SUPPORT = False

class LaplaceApproximation(unittest.TestCase):

    def setUp(self):
        npr.seed(1)
        self.tol = 1e-8

    def compute_laplace(self, setup, test):
        import numpy as np
        import numpy.linalg as npla
        import TransportMaps as TM

        distribution = test['target_distribution']

        # Build Laplace approximation
        d_laplace = TM.laplace_approximation(distribution, None, tol=self.tol)

        # Compare mean
        self.assertTrue( npla.norm(distribution.mu - d_laplace.mu) <= self.tol )

        # Compute KL divergence between the two
        qtype = 3
        qparams = [10]*distribution.dim
        kldiv = TM.kl_divergence(distribution, d_laplace, None, None,
                                 qtype=qtype, qparams=qparams)
        self.assertTrue( np.abs(kldiv) <= 1e-2 )

    def test_linear1d(self):
        import TransportMaps.tests.TestFunctions as TF
        title, setup, Tparams = TF.get(0)
        self.compute_laplace(setup, Tparams)

    def test_linear2d(self):
        import TransportMaps.tests.TestFunctions as TF
        title, setup, Tparams = TF.get(9)
        self.compute_laplace(setup, Tparams)

def build_suite(ttype='all'):
    suite_laplace_approx = unittest.TestLoader().loadTestsFromTestCase( LaplaceApproximation )
    # GROUP SUITES
    suites_list = []
    if ttype in ['all','serial']:
        suites_list = [ suite_laplace_approx ]
    all_suites = unittest.TestSuite( suites_list )
    return all_suites


def run_tests(ttype='all'):
    all_suites = build_suite(ttype)
    # RUN
    unittest.TextTestRunner(verbosity=2).run(all_suites)

if __name__ == '__main__':
    run_tests()



#
# This file is part of TransportMaps.
#
# TransportMaps is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# TransportMaps is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with TransportMaps.  If not, see <http://www.gnu.org/licenses/>.
#
# Transport Maps Library
# Copyright (C) 2015-2017 Massachusetts Institute of Technology
# Uncertainty Quantification group
# Department of Aeronautics and Astronautics
#
# Author: Transport Map Team
# Website: transportmaps.mit.edu
# Support: transportmaps.mit.edu/qa/
#

import unittest
import numpy as np
import numpy.random as npr

try:
    import mpi_map
    MPI_SUPPORT = True
except:
    MPI_SUPPORT = False

class TransportMapDistribution_DerivativeChecks(unittest.TestCase):

    def setUp(self):
        npr.seed(1)
        self.fd_eps = 1e-4

    def test_log_pdf(self):
        import TransportMaps.FiniteDifference as FD

        d = self.distribution
        coeffs = self.coeffs
        params = {}
        params['params_pi'] = None
        params['x'] = self.x

        def pdf(a, params):
            d.coeffs = a
            x = params['x']
            params_pi = params['params_pi']
            out = d.pdf(x, params_pi)
            return out

        def log_pdf(a, params):
            d.coeffs = a
            x = params['x']
            params_pi = params['params_pi']
            out = d.log_pdf(x, params_pi)
            return out

        # Check log_pdf = log(pdf)
        maxerr = np.max( np.abs( pdf(coeffs, params) - np.exp(log_pdf(coeffs, params)) ) )
        self.assertTrue( maxerr <= 1e-10 )

    def test_grad_a_log_pdf(self):
        import TransportMaps.FiniteDifference as FD

        d = self.distribution
        coeffs = self.coeffs
        params = {}
        params['params_pi'] = None
        params['x'] = self.x

        def log_pdf(a, params):
            d.coeffs = a
            x = params['x']
            params_pi = params['params_pi']
            out = d.log_pdf(x, params_pi)
            return out

        def grad_a_log_pdf(a, params):
            d.coeffs = a
            x = params['x']
            params_pi = params['params_pi']
            out = d.grad_a_log_pdf(x, params_pi)
            return out

        flag = FD.check_grad_a(log_pdf, grad_a_log_pdf, coeffs, self.fd_eps,
                               params, verbose=False)
        self.assertTrue( flag )

    def test_tuple_grad_a_log_pdf(self):
        import TransportMaps.FiniteDifference as FD

        d = self.distribution
        coeffs = self.coeffs
        params = {}
        params['params_pi'] = None
        params['x'] = self.x

        def log_pdf(a, params):
            d.coeffs = a
            x = params['x']
            params_pi = params['params_pi']
            out,_ = d.tuple_grad_a_log_pdf(x, params_pi)
            return out

        def grad_a_log_pdf(a, params):
            d.coeffs = a
            x = params['x']
            params_pi = params['params_pi']
            _, out = d.tuple_grad_a_log_pdf(x, params_pi)
            return out

        flag = FD.check_grad_a(log_pdf, grad_a_log_pdf, coeffs, self.fd_eps,
                               params, verbose=False)
        self.assertTrue( flag )


    def test_hess_a_log_pdf(self):
        import TransportMaps.FiniteDifference as FD

        d = self.distribution
        coeffs = self.coeffs
        params = {}
        params['params_pi'] = None
        params['x'] = self.x

        def grad_a_log_pdf(a, params):
            d.coeffs = a
            x = params['x']
            params_pi = params['params_pi']
            out = d.grad_a_log_pdf(x, params_pi)
            return out

        def hess_a_log_pdf(a, params):
            d.coeffs = a
            x = params['x']
            params_pi = params['params_pi']
            out = d.hess_a_log_pdf(x, params_pi)
            return out    

        flag = FD.check_hess_a_from_grad_a(grad_a_log_pdf,
                                           hess_a_log_pdf, coeffs,
                                           self.fd_eps,
                                           params, verbose=False)
        self.assertTrue( flag )

    def test_grad_x_log_pdf(self):
        import TransportMaps.FiniteDifference as FD

        d = self.distribution
        coeffs = self.coeffs
        d.coeffs = coeffs
        params = {}
        params['params_pi'] = None

        def log_pdf(x, params):
            params_pi = params['params_pi']
            out = d.log_pdf(x, params_pi)
            return out

        def grad_x_log_pdf(x, params):
            params_pi = params['params_pi']
            out = d.grad_x_log_pdf(x, params_pi)
            return out

        flag = FD.check_grad_x(log_pdf, grad_x_log_pdf, self.x, self.fd_eps,
                               params, verbose=False)
        self.assertTrue( flag )

    def test_hess_x_log_pdf(self):
        import TransportMaps.FiniteDifference as FD

        d = self.distribution
        coeffs = self.coeffs
        d.coeffs = coeffs
        params = {}
        params['params_pi'] = None

        def grad_x_log_pdf(x, params):
            params_pi = params['params_pi']
            out = d.grad_x_log_pdf(x, params_pi)
            return out

        def hess_x_log_pdf(x, params):
            params_pi = params['params_pi']
            out = d.hess_x_log_pdf(x, params_pi)
            return out

        flag = FD.check_hess_x_from_grad_x(grad_x_log_pdf, hess_x_log_pdf,
                                           self.x, self.fd_eps,
                                           params, verbose=False)
        self.assertTrue( flag )

class PullBackTMD_DerivativeChecks(TransportMapDistribution_DerivativeChecks):
    def setUp(self):
        import TransportMaps.Distributions as DIST
        self.distribution = DIST.PullBackTransportMapDistribution( self.tm_approx,
                                                         self.distribution_pi )
        super(PullBackTMD_DerivativeChecks,self).setUp()

class PushForwardTMD_DerivativeChecks(TransportMapDistribution_DerivativeChecks):
    def setUp(self):
        import TransportMaps.Distributions as DIST
        self.distribution = DIST.PushForwardTransportMapDistribution( self.tm_approx,
                                                            self.distribution_pi )
        super(PushForwardTMD_DerivativeChecks,self).setUp()

    @unittest.skip("Not Implemented")
    def test_tuple_grad_a_log_pdf(self):
        pass
        
    @unittest.skip("Not Implemented")
    def test_hess_a_log_pdf(self):
        pass

    @unittest.skip("Not Implemented")
    def test_grad_x_log_pdf(self):
        pass

    @unittest.skip("Not Implemented")
    def test_hess_x_log_pdf(self):
        pass

class IntegratedExponential:
    def setUpApprox(self):
        import numpy.random as npr
        import SpectralToolbox.Spectral1D as S1D
        import TransportMaps as TM
        import TransportMaps.Functionals as FUNC
        import TransportMaps.Maps as MAPS
        # Build the transport map (isotropic for each entry)
        self.order = 5
        approx_list = []
        active_vars = []
        for i in range(self.dim):
            c_basis_list = [S1D.HermiteProbabilistsPolynomial()] * (i+1)
            c_orders_list = ([self.order] * i) + [0]
            c_approx = FUNC.LinearSpanApproximation(c_basis_list, spantype='full',
                                                             order_list=c_orders_list)
            e_basis_list = [S1D.ConstantExtendedHermiteProbabilistsFunction()] * (i+1)
            e_orders_list = [self.order] * (i+1)
            e_approx = FUNC.LinearSpanApproximation(e_basis_list, spantype='full',
                                                             order_list=e_orders_list)
            approx = FUNC.MonotonicIntegratedExponentialApproximation(c_approx, e_approx)
            approx_list.append( approx )
            active_vars.append( range(i+1) )
        self.tm_approx = MAPS.IntegratedExponentialTriangularTransportMap(active_vars,
                                                                          approx_list)
        self.params = {}
        self.params['params_t'] = None
        self.coeffs = np.zeros( self.tm_approx.n_coeffs )
        # self.coeffs = npr.randn(self.tm_approx.n_coeffs) / 100.

class IntegratedExponentialPBTMD_DerivativeChecks(IntegratedExponential,
                                                  PullBackTMD_DerivativeChecks):
    def setUp(self):
        super(IntegratedExponentialPBTMD_DerivativeChecks, self).setUpApprox()
        super(IntegratedExponentialPBTMD_DerivativeChecks, self).setUp()

class IntegratedExponentialPFTMD_DerivativeChecks(IntegratedExponential,
                                                  PushForwardTMD_DerivativeChecks):
    def setUp(self):
        super(IntegratedExponentialPFTMD_DerivativeChecks, self).setUpApprox()
        super(IntegratedExponentialPFTMD_DerivativeChecks, self).setUp()

class CommonBasisIntegratedExponential:
    def setUpApprox(self):
        import numpy.random as npr
        import SpectralToolbox.Spectral1D as S1D
        import TransportMaps as TM
        import TransportMaps.Functionals as FUNC
        import TransportMaps.Maps as MAPS
        # Build the transport map (isotropic for each entry)
        self.order = 5
        approx_list = []
        active_vars = []
        c_basis_list = [S1D.HermiteProbabilistsPolynomial() for i in range(self.dim)]
        e_basis_list = [S1D.ConstantExtendedHermiteProbabilistsFunction() for i in range(self.dim)]
        for i in range(self.dim):
            c_orders_list = ([self.order] * i) + [0]
            c_approx = FUNC.LinearSpanApproximation(c_basis_list[:i+1], spantype='full',
                                                             order_list=c_orders_list)
            e_orders_list = [self.order] * (i+1)
            e_approx = FUNC.LinearSpanApproximation(e_basis_list[:i+1], spantype='full',
                                                             order_list=e_orders_list)
            approx = FUNC.MonotonicIntegratedExponentialApproximation(c_approx, e_approx)
            approx_list.append( approx )
            active_vars.append( range(i+1) )
        self.tm_approx = MAPS.CommonBasisIntegratedExponentialTriangularTransportMap(active_vars,
                                                                                     approx_list)
        self.params = {}
        self.params['params_t'] = None
        self.coeffs = np.zeros( self.tm_approx.n_coeffs )
        # self.coeffs = npr.randn(self.tm_approx.n_coeffs) / 100. 

class CommonBasisIntegratedExponentialPBTMD_DerivativeChecks(CommonBasisIntegratedExponential,
                                                             PullBackTMD_DerivativeChecks):
    def setUp(self):
        super(CommonBasisIntegratedExponentialPBTMD_DerivativeChecks, self).setUpApprox()
        super(CommonBasisIntegratedExponentialPBTMD_DerivativeChecks, self).setUp()

class CommonBasisIntegratedExponentialPFTMD_DerivativeChecks(CommonBasisIntegratedExponential,
                                                             PushForwardTMD_DerivativeChecks):
    def setUp(self):
        super(CommonBasisIntegratedExponentialPFTMD_DerivativeChecks, self).setUpApprox()
        super(CommonBasisIntegratedExponentialPFTMD_DerivativeChecks, self).setUp()

class TotOrdIntegratedExponential:
    def setUpApprox(self):
        import numpy.random as npr
        import SpectralToolbox.Spectral1D as S1D
        import TransportMaps as TM
        import TransportMaps.Functionals as FUNC
        import TransportMaps.Maps as MAPS
        # Build the transport map (isotropic for each entry)
        self.order = 5
        approx_list = []
        active_vars = []
        for i in range(self.dim):
            c_basis_list = [S1D.HermiteProbabilistsPolynomial()] * (i+1)
            c_orders_list = ([self.order] * i) + [0]
            c_approx = FUNC.LinearSpanApproximation(c_basis_list, spantype='total',
                                                             order_list=c_orders_list)
            e_basis_list = [S1D.ConstantExtendedHermiteProbabilistsFunction()] * (i+1)
            e_orders_list = [self.order] * (i+1)
            e_approx = FUNC.LinearSpanApproximation(e_basis_list, spantype='total',
                                                             order_list=e_orders_list)
            approx = FUNC.MonotonicIntegratedExponentialApproximation(c_approx, e_approx)
            approx_list.append( approx )
            active_vars.append( range(i+1) )
        self.tm_approx = MAPS.IntegratedExponentialTriangularTransportMap(active_vars,
                                                                          approx_list)
        self.params = {}
        self.params['params_t'] = None
        self.coeffs = np.zeros(self.tm_approx.n_coeffs)
        # self.coeffs = npr.randn(self.tm_approx.n_coeffs) / 100.

class TotOrdIntegratedExponentialPBTMD_DerivativeChecks(TotOrdIntegratedExponential,
                                                        PullBackTMD_DerivativeChecks):
    def setUp(self):
        super(TotOrdIntegratedExponentialPBTMD_DerivativeChecks, self).setUpApprox()
        super(TotOrdIntegratedExponentialPBTMD_DerivativeChecks, self).setUp()

class TotOrdIntegratedExponentialPFTMD_DerivativeChecks(TotOrdIntegratedExponential,
                                                        PushForwardTMD_DerivativeChecks):
    def setUp(self):
        super(TotOrdIntegratedExponentialPFTMD_DerivativeChecks, self).setUpApprox()
        super(TotOrdIntegratedExponentialPFTMD_DerivativeChecks, self).setUp()

class CommonBasisTotOrdIntegratedExponential:
    def setUpApprox(self):
        import numpy.random as npr
        import SpectralToolbox.Spectral1D as S1D
        import TransportMaps as TM
        import TransportMaps.Functionals as FUNC
        import TransportMaps.Maps as MAPS
        # Build the transport map (isotropic for each entry)
        self.order = 5
        approx_list = []
        active_vars = []
        c_basis_list = [S1D.HermiteProbabilistsPolynomial() for i in range(self.dim)]
        e_basis_list = [S1D.ConstantExtendedHermiteProbabilistsFunction() for i in range(self.dim)]
        for i in range(self.dim):
            c_orders_list = ([self.order] * i) + [0]
            c_approx = FUNC.LinearSpanApproximation(c_basis_list[:i+1], spantype='total',
                                                             order_list=c_orders_list)
            e_orders_list = [self.order] * (i+1)
            e_approx = FUNC.LinearSpanApproximation(e_basis_list[:i+1], spantype='total',
                                                             order_list=e_orders_list)
            approx = FUNC.MonotonicIntegratedExponentialApproximation(c_approx, e_approx)
            approx_list.append( approx )
            active_vars.append( range(i+1) )
        self.tm_approx = MAPS.CommonBasisIntegratedExponentialTriangularTransportMap(active_vars,
                                                                                     approx_list)
        self.params = {}
        self.params['params_t'] = None
        self.coeffs = np.zeros(self.tm_approx.n_coeffs)
        # self.coeffs = npr.randn(self.tm_approx.n_coeffs) / 100.

class CommonBasisTotOrdIntegratedExponentialPBTMD_DerivativeChecks(
        CommonBasisTotOrdIntegratedExponential,
        PullBackTMD_DerivativeChecks):
    def setUp(self):
        super(CommonBasisTotOrdIntegratedExponentialPBTMD_DerivativeChecks, self).setUpApprox()
        super(CommonBasisTotOrdIntegratedExponentialPBTMD_DerivativeChecks, self).setUp()

class CommonBasisTotOrdIntegratedExponentialPFTMD_DerivativeChecks(
        CommonBasisTotOrdIntegratedExponential,
        PushForwardTMD_DerivativeChecks):
    def setUp(self):
        super(CommonBasisTotOrdIntegratedExponentialPFTMD_DerivativeChecks, self).setUpApprox()
        super(CommonBasisTotOrdIntegratedExponentialPFTMD_DerivativeChecks, self).setUp()

class IntegratedSquared:
    def setUpApprox(self):
        import numpy.random as npr
        import SpectralToolbox.Spectral1D as S1D
        import TransportMaps as TM
        import TransportMaps.Functionals as FUNC
        import TransportMaps.Maps as MAPS
        # Build the transport map (isotropic for each entry)
        self.order = 3
        approx_list = []
        active_vars = []
        for i in range(self.dim):
            c_basis_list = [S1D.HermiteProbabilistsPolynomial()] * (i+1)
            c_orders_list = ([self.order] * i) + [0]
            c_approx = FUNC.LinearSpanApproximation(c_basis_list, spantype='full',
                                                             order_list=c_orders_list)
            e_basis_list = [S1D.ConstantExtendedHermiteProbabilistsFunction()] * (i+1)
            e_orders_list = [self.order] * (i+1)
            e_approx = FUNC.LinearSpanApproximation(e_basis_list, spantype='full',
                                                             order_list=e_orders_list)
            approx = FUNC.MonotonicIntegratedSquaredApproximation(c_approx, e_approx)
            approx_list.append( approx )
            active_vars.append( range(i+1) )
        self.tm_approx = MAPS.IntegratedSquaredTriangularTransportMap(active_vars,
                                                                      approx_list)
        self.params = {}
        self.params['params_t'] = None
        coeffs = []
        for approx in self.tm_approx.approx_list:
            # coeffs.append( npr.random(approx.c.n_coeffs)/10. )
            # coeffs.append( npr.random(approx.h.n_coeffs) )
            coeffs.append( npr.randn(approx.c.n_coeffs) / 10. )
            h_coeffs = npr.randn(approx.h.n_coeffs) / 10.
            midx = approx.h.get_multi_idxs()
            idx0 = midx.index( tuple([0]*approx.dim) )
            h_coeffs[idx0] += 1.
            coeffs.append( h_coeffs )
        self.coeffs = np.hstack(coeffs)

class IntegratedSquaredPBTMD_DerivativeChecks(IntegratedSquared,
                                              PullBackTMD_DerivativeChecks):
    def setUp(self):
        super(IntegratedSquaredPBTMD_DerivativeChecks, self).setUpApprox()
        super(IntegratedSquaredPBTMD_DerivativeChecks, self).setUp()

class IntegratedSquaredPFTMD_DerivativeChecks(IntegratedSquared,
                                              PushForwardTMD_DerivativeChecks):
    def setUp(self):
        super(IntegratedSquaredPFTMD_DerivativeChecks, self).setUpApprox()
        super(IntegratedSquaredPFTMD_DerivativeChecks, self).setUp()

class TotOrdIntegratedSquared:
    def setUpApprox(self):
        import numpy.random as npr
        import SpectralToolbox.Spectral1D as S1D
        import TransportMaps as TM
        import TransportMaps.Functionals as FUNC
        import TransportMaps.Maps as MAPS
        # Build the transport map (isotropic for each entry)
        self.order = 3
        approx_list = []
        active_vars = []
        for i in range(self.dim):
            c_basis_list = [S1D.HermiteProbabilistsPolynomial()] * (i+1)
            c_orders_list = ([self.order] * i) + [0]
            c_approx = FUNC.LinearSpanApproximation(c_basis_list, spantype='total',
                                                             order_list=c_orders_list)
            e_basis_list = [S1D.ConstantExtendedHermiteProbabilistsFunction()] * (i+1)
            e_orders_list = [self.order] * (i+1)
            e_approx = FUNC.LinearSpanApproximation(e_basis_list, spantype='total',
                                                             order_list=e_orders_list)
            approx = FUNC.MonotonicIntegratedSquaredApproximation(c_approx, e_approx)
            approx_list.append( approx )
            active_vars.append( range(i+1) )
        self.tm_approx = MAPS.IntegratedSquaredTriangularTransportMap(active_vars,
                                                                          approx_list)
        self.params = {}
        self.params['params_t'] = None
        coeffs = []
        for approx in self.tm_approx.approx_list:
            # coeffs.append( npr.random(approx.c.n_coeffs)/10. )
            # coeffs.append( npr.random(approx.h.n_coeffs) )
            coeffs.append( npr.randn(approx.c.n_coeffs) / 10. )
            h_coeffs = npr.randn(approx.h.n_coeffs) / 10.
            midx = approx.h.get_multi_idxs()
            idx0 = midx.index( tuple([0]*approx.dim) )
            h_coeffs[idx0] += 1.
            coeffs.append( h_coeffs )
        self.coeffs = np.hstack(coeffs)

class TotOrdIntegratedSquaredPBTMD_DerivativeChecks(TotOrdIntegratedSquared,
                                                    PullBackTMD_DerivativeChecks):
    def setUp(self):
        super(TotOrdIntegratedSquaredPBTMD_DerivativeChecks, self).setUpApprox()
        super(TotOrdIntegratedSquaredPBTMD_DerivativeChecks, self).setUp()

class TotOrdIntegratedSquaredPFTMD_DerivativeChecks(TotOrdIntegratedSquared,
                                                    PushForwardTMD_DerivativeChecks):
    def setUp(self):
        super(TotOrdIntegratedSquaredPFTMD_DerivativeChecks, self).setUpApprox()
        super(TotOrdIntegratedSquaredPFTMD_DerivativeChecks, self).setUp()
        
class LinearSpanPBTMD_DerivativeChecks(PullBackTMD_DerivativeChecks):

    def setUp(self):
        import SpectralToolbox.Spectral1D as S1D
        import TransportMaps as TM
        import TransportMaps.Functionals as FUNC
        import TransportMaps.Maps as MAPS
        # Build the transport map (isotropic for each entry)
        self.order = 5
        approx_list = []
        active_vars = []
        for i in range(self.dim):
            basis_list = [S1D.HermiteProbabilistsPolynomial()] * (i+1)
            orders_list = [self.order] * (i+1)
            approx = FUNC.MonotonicLinearSpanApproximation(basis_list, spantype='full',
                                                           order_list=orders_list)
            approx_list.append( approx )
            active_vars.append( range(i+1) )
        self.tm_approx = MAPS.LinearSpanTriangularTransportMap(active_vars,
                                                              approx_list)
        self.params = {}
        self.params['params_t'] = None
        # Set coefficients for linear map
        self.coeffs = np.zeros(self.tm_approx.n_coeffs)
        self.coeffs[1] = 1.
        idx = (self.order+1)
        for d in range(2,self.dim+1):
            self.coeffs[ idx + 1 ] = 1.
            idx += (self.order+1)**d
        super(LinearSpanPBTMD_DerivativeChecks, self).setUp()

class CommonBasisLinearSpanPBTMD_DerivativeChecks(PullBackTMD_DerivativeChecks):

    def setUp(self):
        import SpectralToolbox.Spectral1D as S1D
        import TransportMaps as TM
        import TransportMaps.Functionals as FUNC
        import TransportMaps.Maps as MAPS
        # Build the transport map (isotropic for each entry)
        self.order = 5
        approx_list = []
        active_vars = []
        basis_list = [S1D.HermiteProbabilistsPolynomial() for i in range(self.dim)]
        for i in range(self.dim):
            orders_list = [self.order] * (i+1)
            approx = FUNC.MonotonicLinearSpanApproximation(basis_list[:i+1], spantype='full',
                                                           order_list=orders_list)
            approx_list.append( approx )
            active_vars.append( range(i+1) )
        self.tm_approx = MAPS.CommonBasisLinearSpanTriangularTransportMap(active_vars,
                                                                          approx_list)
        self.params = {}
        self.params['params_t'] = None
        # Set coefficients for linear map
        self.coeffs = np.zeros(self.tm_approx.n_coeffs)
        self.coeffs[1] = 1.
        idx = (self.order+1)
        for d in range(2,self.dim+1):
            self.coeffs[ idx + 1 ] = 1.
            idx += (self.order+1)**d
        super(CommonBasisLinearSpanPBTMD_DerivativeChecks, self).setUp()

class TMD_TestCase(object):

    def setUp_test_case(self):
        import TransportMaps.Distributions as DIST
        self.dim = self.setup['dim']
        self.target_distribution = self.Tparams['target_distribution']
        self.support_map = self.Tparams['support_map']
        self.distribution_pi = DIST.PullBackTransportMapDistribution( self.support_map,
                                                            self.target_distribution )
        self.base_distribution = self.Tparams['base_distribution']
        self.qtype = 0
        self.qparams = 5
        (self.x, self.w) = self.base_distribution.quadrature(self.qtype, self.qparams)

class Linear1D_TMD_TestCase(TMD_TestCase):    
    def setUp_test_case(self):
        import TransportMaps.tests.TestFunctions as TF
        title, self.setup, self.Tparams = TF.get(0)
        super(Linear1D_TMD_TestCase,self).setUp_test_case()

class ArcTan1D_TMD_TestCase(TMD_TestCase):
    def setUp_test_case(self):
        import TransportMaps.tests.TestFunctions as TF
        title, self.setup, self.Tparams = TF.get(2)
        super(ArcTan1D_TMD_TestCase,self).setUp_test_case()

class Exp1D_TMD_TestCase(TMD_TestCase):
    def setUp_test_case(self):
        import TransportMaps.tests.TestFunctions as TF
        title, self.setup, self.Tparams = TF.get(3)
        super(Exp1D_TMD_TestCase,self).setUp_test_case()

class Logistic1D_TMD_TestCase(TMD_TestCase):
    def setUp_test_case(self):
        import TransportMaps.tests.TestFunctions as TF
        title, self.setup, self.Tparams = TF.get(4)
        super(Logistic1D_TMD_TestCase,self).setUp_test_case()

class Gamma1D_TMD_TestCase(TMD_TestCase):
    def setUp_test_case(self):
        import TransportMaps.tests.TestFunctions as TF
        title, self.setup, self.Tparams = TF.get(5)
        super(Gamma1D_TMD_TestCase,self).setUp_test_case()

class Beta1D_TMD_TestCase(TMD_TestCase):
    def setUp_test_case(self):
        import TransportMaps.tests.TestFunctions as TF
        title, self.setup, self.Tparams = TF.get(6)
        super(Beta1D_TMD_TestCase,self).setUp_test_case()

class Gumbel1D_TMD_TestCase(TMD_TestCase):
    def setUp_test_case(self):
        import TransportMaps.tests.TestFunctions as TF
        title, self.setup, self.Tparams = TF.get(7)
        super(Gumbel1D_TMD_TestCase,self).setUp_test_case()

class Linear2D_TMD_TestCase(TMD_TestCase):
    def setUp_test_case(self):
        import TransportMaps.tests.TestFunctions as TF
        title, self.setup, self.Tparams = TF.get(9)
        super(Linear2D_TMD_TestCase,self).setUp_test_case()

class Banana2D_TMD_TestCase(TMD_TestCase):
    def setUp_test_case(self):
        import TransportMaps.tests.TestFunctions as TF
        title, self.setup, self.Tparams = TF.get(10)
        super(Banana2D_TMD_TestCase,self).setUp_test_case()

# INTEGRATED EXPONENTIAL PULLBACK
class Linear1D_IEPBTMD_DerivativeChecks(Linear1D_TMD_TestCase,
                                        IntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Linear1D_IEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear1D_IEPBTMD_DerivativeChecks,self).setUp()

class ArcTan1D_IEPBTMD_DerivativeChecks(ArcTan1D_TMD_TestCase,
                                        IntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(ArcTan1D_IEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(ArcTan1D_IEPBTMD_DerivativeChecks,self).setUp()

class Exp1D_IEPBTMD_DerivativeChecks(Exp1D_TMD_TestCase,
                                     IntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Exp1D_IEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Exp1D_IEPBTMD_DerivativeChecks,self).setUp()

class Logistic1D_IEPBTMD_DerivativeChecks(Logistic1D_TMD_TestCase,
                                          IntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Logistic1D_IEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Logistic1D_IEPBTMD_DerivativeChecks,self).setUp()

class Gamma1D_IEPBTMD_DerivativeChecks(Gamma1D_TMD_TestCase,
                                       IntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Gamma1D_IEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Gamma1D_IEPBTMD_DerivativeChecks,self).setUp()

class Beta1D_IEPBTMD_DerivativeChecks(Beta1D_TMD_TestCase,
                                      IntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Beta1D_IEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Beta1D_IEPBTMD_DerivativeChecks,self).setUp()

class Gumbel1D_IEPBTMD_DerivativeChecks(Gumbel1D_TMD_TestCase,
                                        IntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Gumbel1D_IEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Gumbel1D_IEPBTMD_DerivativeChecks,self).setUp()

class Linear2D_IEPBTMD_DerivativeChecks(Linear2D_TMD_TestCase,
                                        IntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Linear2D_IEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear2D_IEPBTMD_DerivativeChecks,self).setUp()

class Banana2D_IEPBTMD_DerivativeChecks(Banana2D_TMD_TestCase,
                                        IntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Banana2D_IEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Banana2D_IEPBTMD_DerivativeChecks,self).setUp()

# COMMON BASIS INTEGRATED EXPONENTIAL PULLBACK
class Linear1D_CBIEPBTMD_DerivativeChecks(Linear1D_TMD_TestCase,
                                          CommonBasisIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Linear1D_CBIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear1D_CBIEPBTMD_DerivativeChecks,self).setUp()

class ArcTan1D_CBIEPBTMD_DerivativeChecks(ArcTan1D_TMD_TestCase,
                                          CommonBasisIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(ArcTan1D_CBIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(ArcTan1D_CBIEPBTMD_DerivativeChecks,self).setUp()

class Exp1D_CBIEPBTMD_DerivativeChecks(Exp1D_TMD_TestCase,
                                       CommonBasisIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Exp1D_CBIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Exp1D_CBIEPBTMD_DerivativeChecks,self).setUp()

class Logistic1D_CBIEPBTMD_DerivativeChecks(Logistic1D_TMD_TestCase,
                                            CommonBasisIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Logistic1D_CBIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Logistic1D_CBIEPBTMD_DerivativeChecks,self).setUp()

class Gamma1D_CBIEPBTMD_DerivativeChecks(Gamma1D_TMD_TestCase,
                                         CommonBasisIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Gamma1D_CBIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Gamma1D_CBIEPBTMD_DerivativeChecks,self).setUp()

class Beta1D_CBIEPBTMD_DerivativeChecks(Beta1D_TMD_TestCase,
                                        CommonBasisIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Beta1D_CBIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Beta1D_CBIEPBTMD_DerivativeChecks,self).setUp()

class Gumbel1D_CBIEPBTMD_DerivativeChecks(Gumbel1D_TMD_TestCase,
                                          CommonBasisIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Gumbel1D_CBIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Gumbel1D_CBIEPBTMD_DerivativeChecks,self).setUp()

class Linear2D_CBIEPBTMD_DerivativeChecks(Linear2D_TMD_TestCase,
                                          CommonBasisIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Linear2D_CBIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear2D_CBIEPBTMD_DerivativeChecks,self).setUp()

class Banana2D_CBIEPBTMD_DerivativeChecks(Banana2D_TMD_TestCase,
                                          CommonBasisIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Banana2D_CBIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Banana2D_CBIEPBTMD_DerivativeChecks,self).setUp()

# TOTAL ORDER INTEGRATED EXPONENTIAL PULLBACK
class Linear1D_TOIEPBTMD_DerivativeChecks(Linear1D_TMD_TestCase,
                                        TotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Linear1D_TOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear1D_TOIEPBTMD_DerivativeChecks,self).setUp()

class ArcTan1D_TOIEPBTMD_DerivativeChecks(ArcTan1D_TMD_TestCase,
                                        TotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(ArcTan1D_TOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(ArcTan1D_TOIEPBTMD_DerivativeChecks,self).setUp()

class Exp1D_TOIEPBTMD_DerivativeChecks(Exp1D_TMD_TestCase,
                                     TotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Exp1D_TOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Exp1D_TOIEPBTMD_DerivativeChecks,self).setUp()

class Logistic1D_TOIEPBTMD_DerivativeChecks(Logistic1D_TMD_TestCase,
                                          TotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Logistic1D_TOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Logistic1D_TOIEPBTMD_DerivativeChecks,self).setUp()

class Gamma1D_TOIEPBTMD_DerivativeChecks(Gamma1D_TMD_TestCase,
                                       TotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Gamma1D_TOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Gamma1D_TOIEPBTMD_DerivativeChecks,self).setUp()

class Beta1D_TOIEPBTMD_DerivativeChecks(Beta1D_TMD_TestCase,
                                      TotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Beta1D_TOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Beta1D_TOIEPBTMD_DerivativeChecks,self).setUp()

class Gumbel1D_TOIEPBTMD_DerivativeChecks(Gumbel1D_TMD_TestCase,
                                        TotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Gumbel1D_TOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Gumbel1D_TOIEPBTMD_DerivativeChecks,self).setUp()

class Linear2D_TOIEPBTMD_DerivativeChecks(Linear2D_TMD_TestCase,
                                        TotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Linear2D_TOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear2D_TOIEPBTMD_DerivativeChecks,self).setUp()

class Banana2D_TOIEPBTMD_DerivativeChecks(Banana2D_TMD_TestCase,
                                        TotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Banana2D_TOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Banana2D_TOIEPBTMD_DerivativeChecks,self).setUp()

# COMMON BASIS TOTAL ORDER INTEGRATED EXPONENTIAL PULLBACK
class Linear1D_CBTOIEPBTMD_DerivativeChecks(
        Linear1D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Linear1D_CBTOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear1D_CBTOIEPBTMD_DerivativeChecks,self).setUp()

class ArcTan1D_CBTOIEPBTMD_DerivativeChecks(
        ArcTan1D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(ArcTan1D_CBTOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(ArcTan1D_CBTOIEPBTMD_DerivativeChecks,self).setUp()

class Exp1D_CBTOIEPBTMD_DerivativeChecks(
        Exp1D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Exp1D_CBTOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Exp1D_CBTOIEPBTMD_DerivativeChecks,self).setUp()

class Logistic1D_CBTOIEPBTMD_DerivativeChecks(
        Logistic1D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Logistic1D_CBTOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Logistic1D_CBTOIEPBTMD_DerivativeChecks,self).setUp()

class Gamma1D_CBTOIEPBTMD_DerivativeChecks(
        Gamma1D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Gamma1D_CBTOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Gamma1D_CBTOIEPBTMD_DerivativeChecks,self).setUp()

class Beta1D_CBTOIEPBTMD_DerivativeChecks(
        Beta1D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Beta1D_CBTOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Beta1D_CBTOIEPBTMD_DerivativeChecks,self).setUp()

class Gumbel1D_CBTOIEPBTMD_DerivativeChecks(
        Gumbel1D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Gumbel1D_CBTOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Gumbel1D_CBTOIEPBTMD_DerivativeChecks,self).setUp()

class Linear2D_CBTOIEPBTMD_DerivativeChecks(
        Linear2D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Linear2D_CBTOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear2D_CBTOIEPBTMD_DerivativeChecks,self).setUp()

class Banana2D_CBTOIEPBTMD_DerivativeChecks(
        Banana2D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPBTMD_DerivativeChecks):
    def setUp(self):
        super(Banana2D_CBTOIEPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Banana2D_CBTOIEPBTMD_DerivativeChecks,self).setUp()

# INTEGRATED EXPONENTIAL PUSHFORWARD
class Linear1D_IEPFTMD_DerivativeChecks(Linear1D_TMD_TestCase,
                                        IntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Linear1D_IEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear1D_IEPFTMD_DerivativeChecks,self).setUp()

class ArcTan1D_IEPFTMD_DerivativeChecks(ArcTan1D_TMD_TestCase,
                                        IntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(ArcTan1D_IEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(ArcTan1D_IEPFTMD_DerivativeChecks,self).setUp()

class Exp1D_IEPFTMD_DerivativeChecks(Exp1D_TMD_TestCase,
                                     IntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Exp1D_IEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Exp1D_IEPFTMD_DerivativeChecks,self).setUp()

class Logistic1D_IEPFTMD_DerivativeChecks(Logistic1D_TMD_TestCase,
                                          IntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Logistic1D_IEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Logistic1D_IEPFTMD_DerivativeChecks,self).setUp()

class Gamma1D_IEPFTMD_DerivativeChecks(Gamma1D_TMD_TestCase,
                                       IntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Gamma1D_IEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Gamma1D_IEPFTMD_DerivativeChecks,self).setUp()

class Beta1D_IEPFTMD_DerivativeChecks(Beta1D_TMD_TestCase,
                                      IntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Beta1D_IEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Beta1D_IEPFTMD_DerivativeChecks,self).setUp()

class Gumbel1D_IEPFTMD_DerivativeChecks(Gumbel1D_TMD_TestCase,
                                        IntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Gumbel1D_IEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Gumbel1D_IEPFTMD_DerivativeChecks,self).setUp()

class Linear2D_IEPFTMD_DerivativeChecks(Linear2D_TMD_TestCase,
                                        IntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Linear2D_IEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear2D_IEPFTMD_DerivativeChecks,self).setUp()

class Banana2D_IEPFTMD_DerivativeChecks(Banana2D_TMD_TestCase,
                                        IntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Banana2D_IEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Banana2D_IEPFTMD_DerivativeChecks,self).setUp()

# COMMON BASIS INTEGRATED EXPONENTIAL PUSHFORWARD
class Linear1D_CBIEPFTMD_DerivativeChecks(Linear1D_TMD_TestCase,
                                          CommonBasisIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Linear1D_CBIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear1D_CBIEPFTMD_DerivativeChecks,self).setUp()

class ArcTan1D_CBIEPFTMD_DerivativeChecks(ArcTan1D_TMD_TestCase,
                                          CommonBasisIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(ArcTan1D_CBIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(ArcTan1D_CBIEPFTMD_DerivativeChecks,self).setUp()

class Exp1D_CBIEPFTMD_DerivativeChecks(Exp1D_TMD_TestCase,
                                       CommonBasisIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Exp1D_CBIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Exp1D_CBIEPFTMD_DerivativeChecks,self).setUp()

class Logistic1D_CBIEPFTMD_DerivativeChecks(Logistic1D_TMD_TestCase,
                                            CommonBasisIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Logistic1D_CBIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Logistic1D_CBIEPFTMD_DerivativeChecks,self).setUp()

class Gamma1D_CBIEPFTMD_DerivativeChecks(Gamma1D_TMD_TestCase,
                                         CommonBasisIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Gamma1D_CBIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Gamma1D_CBIEPFTMD_DerivativeChecks,self).setUp()

class Beta1D_CBIEPFTMD_DerivativeChecks(Beta1D_TMD_TestCase,
                                        CommonBasisIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Beta1D_CBIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Beta1D_CBIEPFTMD_DerivativeChecks,self).setUp()

class Gumbel1D_CBIEPFTMD_DerivativeChecks(Gumbel1D_TMD_TestCase,
                                          CommonBasisIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Gumbel1D_CBIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Gumbel1D_CBIEPFTMD_DerivativeChecks,self).setUp()

class Linear2D_CBIEPFTMD_DerivativeChecks(Linear2D_TMD_TestCase,
                                          CommonBasisIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Linear2D_CBIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear2D_CBIEPFTMD_DerivativeChecks,self).setUp()

class Banana2D_CBIEPFTMD_DerivativeChecks(Banana2D_TMD_TestCase,
                                          CommonBasisIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Banana2D_CBIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Banana2D_CBIEPFTMD_DerivativeChecks,self).setUp()

# TOTAL ORDER INTEGRATED EXPONENTIAL PUSHFORWARD
class Linear1D_TOIEPFTMD_DerivativeChecks(Linear1D_TMD_TestCase,
                                        TotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Linear1D_TOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear1D_TOIEPFTMD_DerivativeChecks,self).setUp()

class ArcTan1D_TOIEPFTMD_DerivativeChecks(ArcTan1D_TMD_TestCase,
                                        TotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(ArcTan1D_TOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(ArcTan1D_TOIEPFTMD_DerivativeChecks,self).setUp()

class Exp1D_TOIEPFTMD_DerivativeChecks(Exp1D_TMD_TestCase,
                                     TotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Exp1D_TOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Exp1D_TOIEPFTMD_DerivativeChecks,self).setUp()

class Logistic1D_TOIEPFTMD_DerivativeChecks(Logistic1D_TMD_TestCase,
                                          TotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Logistic1D_TOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Logistic1D_TOIEPFTMD_DerivativeChecks,self).setUp()

class Gamma1D_TOIEPFTMD_DerivativeChecks(Gamma1D_TMD_TestCase,
                                       TotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Gamma1D_TOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Gamma1D_TOIEPFTMD_DerivativeChecks,self).setUp()

class Beta1D_TOIEPFTMD_DerivativeChecks(Beta1D_TMD_TestCase,
                                      TotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Beta1D_TOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Beta1D_TOIEPFTMD_DerivativeChecks,self).setUp()

class Gumbel1D_TOIEPFTMD_DerivativeChecks(Gumbel1D_TMD_TestCase,
                                        TotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Gumbel1D_TOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Gumbel1D_TOIEPFTMD_DerivativeChecks,self).setUp()

class Linear2D_TOIEPFTMD_DerivativeChecks(Linear2D_TMD_TestCase,
                                        TotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Linear2D_TOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear2D_TOIEPFTMD_DerivativeChecks,self).setUp()

class Banana2D_TOIEPFTMD_DerivativeChecks(Banana2D_TMD_TestCase,
                                        TotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Banana2D_TOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Banana2D_TOIEPFTMD_DerivativeChecks,self).setUp()

# COMMON BASIS TOTAL ORDER INTEGRATED EXPONENTIAL PUSHFORWARD
class Linear1D_CBTOIEPFTMD_DerivativeChecks(
        Linear1D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Linear1D_CBTOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear1D_CBTOIEPFTMD_DerivativeChecks,self).setUp()

class ArcTan1D_CBTOIEPFTMD_DerivativeChecks(
        ArcTan1D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(ArcTan1D_CBTOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(ArcTan1D_CBTOIEPFTMD_DerivativeChecks,self).setUp()

class Exp1D_CBTOIEPFTMD_DerivativeChecks(
        Exp1D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Exp1D_CBTOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Exp1D_CBTOIEPFTMD_DerivativeChecks,self).setUp()

class Logistic1D_CBTOIEPFTMD_DerivativeChecks(
        Logistic1D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Logistic1D_CBTOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Logistic1D_CBTOIEPFTMD_DerivativeChecks,self).setUp()

class Gamma1D_CBTOIEPFTMD_DerivativeChecks(
        Gamma1D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Gamma1D_CBTOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Gamma1D_CBTOIEPFTMD_DerivativeChecks,self).setUp()

class Beta1D_CBTOIEPFTMD_DerivativeChecks(
        Beta1D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Beta1D_CBTOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Beta1D_CBTOIEPFTMD_DerivativeChecks,self).setUp()

class Gumbel1D_CBTOIEPFTMD_DerivativeChecks(
        Gumbel1D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Gumbel1D_CBTOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Gumbel1D_CBTOIEPFTMD_DerivativeChecks,self).setUp()

class Linear2D_CBTOIEPFTMD_DerivativeChecks(
        Linear2D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Linear2D_CBTOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear2D_CBTOIEPFTMD_DerivativeChecks,self).setUp()

class Banana2D_CBTOIEPFTMD_DerivativeChecks(
        Banana2D_TMD_TestCase,
        CommonBasisTotOrdIntegratedExponentialPFTMD_DerivativeChecks):
    def setUp(self):
        super(Banana2D_CBTOIEPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Banana2D_CBTOIEPFTMD_DerivativeChecks,self).setUp()

# INTEGRATED SQUARED PULLBACK
class Linear1D_ISPBTMD_DerivativeChecks(Linear1D_TMD_TestCase,
                                        IntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Linear1D_ISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear1D_ISPBTMD_DerivativeChecks,self).setUp()

class ArcTan1D_ISPBTMD_DerivativeChecks(ArcTan1D_TMD_TestCase,
                                        IntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(ArcTan1D_ISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(ArcTan1D_ISPBTMD_DerivativeChecks,self).setUp()

class Exp1D_ISPBTMD_DerivativeChecks(Exp1D_TMD_TestCase,
                                     IntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Exp1D_ISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Exp1D_ISPBTMD_DerivativeChecks,self).setUp()

class Logistic1D_ISPBTMD_DerivativeChecks(Logistic1D_TMD_TestCase,
                                          IntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Logistic1D_ISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Logistic1D_ISPBTMD_DerivativeChecks,self).setUp()

class Gamma1D_ISPBTMD_DerivativeChecks(Gamma1D_TMD_TestCase,
                                       IntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Gamma1D_ISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Gamma1D_ISPBTMD_DerivativeChecks,self).setUp()

class Beta1D_ISPBTMD_DerivativeChecks(Beta1D_TMD_TestCase,
                                      IntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Beta1D_ISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Beta1D_ISPBTMD_DerivativeChecks,self).setUp()

class Gumbel1D_ISPBTMD_DerivativeChecks(Gumbel1D_TMD_TestCase,
                                        IntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Gumbel1D_ISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Gumbel1D_ISPBTMD_DerivativeChecks,self).setUp()

class Linear2D_ISPBTMD_DerivativeChecks(Linear2D_TMD_TestCase,
                                        IntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Linear2D_ISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear2D_ISPBTMD_DerivativeChecks,self).setUp()

class Banana2D_ISPBTMD_DerivativeChecks(Banana2D_TMD_TestCase,
                                        IntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Banana2D_ISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Banana2D_ISPBTMD_DerivativeChecks,self).setUp()

# # COMMON BASIS INTEGRATED SQUARED PULLBACK
# class Linear1D_CBISPBTMD_DerivativeChecks(Linear1D_TMD_TestCase,
#                                           CommonBasisIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Linear1D_CBISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Linear1D_CBISPBTMD_DerivativeChecks,self).setUp()

# class ArcTan1D_CBISPBTMD_DerivativeChecks(ArcTan1D_TMD_TestCase,
#                                           CommonBasisIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(ArcTan1D_CBISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(ArcTan1D_CBISPBTMD_DerivativeChecks,self).setUp()

# class Exp1D_CBISPBTMD_DerivativeChecks(Exp1D_TMD_TestCase,
#                                        CommonBasisIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Exp1D_CBISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Exp1D_CBISPBTMD_DerivativeChecks,self).setUp()

# class Logistic1D_CBISPBTMD_DerivativeChecks(Logistic1D_TMD_TestCase,
#                                             CommonBasisIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Logistic1D_CBISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Logistic1D_CBISPBTMD_DerivativeChecks,self).setUp()

# class Gamma1D_CBISPBTMD_DerivativeChecks(Gamma1D_TMD_TestCase,
#                                          CommonBasisIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Gamma1D_CBISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Gamma1D_CBISPBTMD_DerivativeChecks,self).setUp()

# class Beta1D_CBISPBTMD_DerivativeChecks(Beta1D_TMD_TestCase,
#                                         CommonBasisIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Beta1D_CBISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Beta1D_CBISPBTMD_DerivativeChecks,self).setUp()

# class Gumbel1D_CBISPBTMD_DerivativeChecks(Gumbel1D_TMD_TestCase,
#                                           CommonBasisIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Gumbel1D_CBISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Gumbel1D_CBISPBTMD_DerivativeChecks,self).setUp()

# class Linear2D_CBISPBTMD_DerivativeChecks(Linear2D_TMD_TestCase,
#                                           CommonBasisIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Linear2D_CBISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Linear2D_CBISPBTMD_DerivativeChecks,self).setUp()

# class Banana2D_CBISPBTMD_DerivativeChecks(Banana2D_TMD_TestCase,
#                                           CommonBasisIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Banana2D_CBISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Banana2D_CBISPBTMD_DerivativeChecks,self).setUp()

# TOTAL ORDER INTEGRATED SQUARED PULLBACK
class Linear1D_TOISPBTMD_DerivativeChecks(Linear1D_TMD_TestCase,
                                        TotOrdIntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Linear1D_TOISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear1D_TOISPBTMD_DerivativeChecks,self).setUp()

class ArcTan1D_TOISPBTMD_DerivativeChecks(ArcTan1D_TMD_TestCase,
                                        TotOrdIntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(ArcTan1D_TOISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(ArcTan1D_TOISPBTMD_DerivativeChecks,self).setUp()

class Exp1D_TOISPBTMD_DerivativeChecks(Exp1D_TMD_TestCase,
                                     TotOrdIntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Exp1D_TOISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Exp1D_TOISPBTMD_DerivativeChecks,self).setUp()

class Logistic1D_TOISPBTMD_DerivativeChecks(Logistic1D_TMD_TestCase,
                                          TotOrdIntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Logistic1D_TOISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Logistic1D_TOISPBTMD_DerivativeChecks,self).setUp()

class Gamma1D_TOISPBTMD_DerivativeChecks(Gamma1D_TMD_TestCase,
                                       TotOrdIntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Gamma1D_TOISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Gamma1D_TOISPBTMD_DerivativeChecks,self).setUp()

class Beta1D_TOISPBTMD_DerivativeChecks(Beta1D_TMD_TestCase,
                                      TotOrdIntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Beta1D_TOISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Beta1D_TOISPBTMD_DerivativeChecks,self).setUp()

class Gumbel1D_TOISPBTMD_DerivativeChecks(Gumbel1D_TMD_TestCase,
                                        TotOrdIntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Gumbel1D_TOISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Gumbel1D_TOISPBTMD_DerivativeChecks,self).setUp()

class Linear2D_TOISPBTMD_DerivativeChecks(Linear2D_TMD_TestCase,
                                        TotOrdIntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Linear2D_TOISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear2D_TOISPBTMD_DerivativeChecks,self).setUp()

class Banana2D_TOISPBTMD_DerivativeChecks(Banana2D_TMD_TestCase,
                                        TotOrdIntegratedSquaredPBTMD_DerivativeChecks):
    def setUp(self):
        super(Banana2D_TOISPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Banana2D_TOISPBTMD_DerivativeChecks,self).setUp()

# # COMMON BASIS TOTAL ORDER INTEGRATED SQUARED PULLBACK
# class Linear1D_CBTOISPBTMD_DerivativeChecks(
#         Linear1D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Linear1D_CBTOISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Linear1D_CBTOISPBTMD_DerivativeChecks,self).setUp()

# class ArcTan1D_CBTOISPBTMD_DerivativeChecks(
#         ArcTan1D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(ArcTan1D_CBTOISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(ArcTan1D_CBTOISPBTMD_DerivativeChecks,self).setUp()

# class Exp1D_CBTOISPBTMD_DerivativeChecks(
#         Exp1D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Exp1D_CBTOISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Exp1D_CBTOISPBTMD_DerivativeChecks,self).setUp()

# class Logistic1D_CBTOISPBTMD_DerivativeChecks(
#         Logistic1D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Logistic1D_CBTOISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Logistic1D_CBTOISPBTMD_DerivativeChecks,self).setUp()

# class Gamma1D_CBTOISPBTMD_DerivativeChecks(
#         Gamma1D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Gamma1D_CBTOISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Gamma1D_CBTOISPBTMD_DerivativeChecks,self).setUp()

# class Beta1D_CBTOISPBTMD_DerivativeChecks(
#         Beta1D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Beta1D_CBTOISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Beta1D_CBTOISPBTMD_DerivativeChecks,self).setUp()

# class Gumbel1D_CBTOISPBTMD_DerivativeChecks(
#         Gumbel1D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Gumbel1D_CBTOISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Gumbel1D_CBTOISPBTMD_DerivativeChecks,self).setUp()

# class Linear2D_CBTOISPBTMD_DerivativeChecks(
#         Linear2D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Linear2D_CBTOISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Linear2D_CBTOISPBTMD_DerivativeChecks,self).setUp()

# class Banana2D_CBTOISPBTMD_DerivativeChecks(
#         Banana2D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPBTMD_DerivativeChecks):
#     def setUp(self):
#         super(Banana2D_CBTOISPBTMD_DerivativeChecks,self).setUp_test_case()
#         super(Banana2D_CBTOISPBTMD_DerivativeChecks,self).setUp()

# INTEGRATED SQUARED PUSHFORWARD
class Linear1D_ISPFTMD_DerivativeChecks(Linear1D_TMD_TestCase,
                                        IntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Linear1D_ISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear1D_ISPFTMD_DerivativeChecks,self).setUp()

class ArcTan1D_ISPFTMD_DerivativeChecks(ArcTan1D_TMD_TestCase,
                                        IntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(ArcTan1D_ISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(ArcTan1D_ISPFTMD_DerivativeChecks,self).setUp()

class Exp1D_ISPFTMD_DerivativeChecks(Exp1D_TMD_TestCase,
                                     IntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Exp1D_ISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Exp1D_ISPFTMD_DerivativeChecks,self).setUp()

class Logistic1D_ISPFTMD_DerivativeChecks(Logistic1D_TMD_TestCase,
                                          IntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Logistic1D_ISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Logistic1D_ISPFTMD_DerivativeChecks,self).setUp()

class Gamma1D_ISPFTMD_DerivativeChecks(Gamma1D_TMD_TestCase,
                                       IntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Gamma1D_ISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Gamma1D_ISPFTMD_DerivativeChecks,self).setUp()

class Beta1D_ISPFTMD_DerivativeChecks(Beta1D_TMD_TestCase,
                                      IntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Beta1D_ISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Beta1D_ISPFTMD_DerivativeChecks,self).setUp()

class Gumbel1D_ISPFTMD_DerivativeChecks(Gumbel1D_TMD_TestCase,
                                        IntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Gumbel1D_ISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Gumbel1D_ISPFTMD_DerivativeChecks,self).setUp()

class Linear2D_ISPFTMD_DerivativeChecks(Linear2D_TMD_TestCase,
                                        IntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Linear2D_ISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear2D_ISPFTMD_DerivativeChecks,self).setUp()

class Banana2D_ISPFTMD_DerivativeChecks(Banana2D_TMD_TestCase,
                                        IntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Banana2D_ISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Banana2D_ISPFTMD_DerivativeChecks,self).setUp()

# # COMMON BASIS INTEGRATED SQUARED PUSHFORWARD
# class Linear1D_CBISPFTMD_DerivativeChecks(Linear1D_TMD_TestCase,
#                                           CommonBasisIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Linear1D_CBISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Linear1D_CBISPFTMD_DerivativeChecks,self).setUp()

# class ArcTan1D_CBISPFTMD_DerivativeChecks(ArcTan1D_TMD_TestCase,
#                                           CommonBasisIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(ArcTan1D_CBISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(ArcTan1D_CBISPFTMD_DerivativeChecks,self).setUp()

# class Exp1D_CBISPFTMD_DerivativeChecks(Exp1D_TMD_TestCase,
#                                        CommonBasisIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Exp1D_CBISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Exp1D_CBISPFTMD_DerivativeChecks,self).setUp()

# class Logistic1D_CBISPFTMD_DerivativeChecks(Logistic1D_TMD_TestCase,
#                                             CommonBasisIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Logistic1D_CBISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Logistic1D_CBISPFTMD_DerivativeChecks,self).setUp()

# class Gamma1D_CBISPFTMD_DerivativeChecks(Gamma1D_TMD_TestCase,
#                                          CommonBasisIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Gamma1D_CBISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Gamma1D_CBISPFTMD_DerivativeChecks,self).setUp()

# class Beta1D_CBISPFTMD_DerivativeChecks(Beta1D_TMD_TestCase,
#                                         CommonBasisIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Beta1D_CBISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Beta1D_CBISPFTMD_DerivativeChecks,self).setUp()

# class Gumbel1D_CBISPFTMD_DerivativeChecks(Gumbel1D_TMD_TestCase,
#                                           CommonBasisIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Gumbel1D_CBISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Gumbel1D_CBISPFTMD_DerivativeChecks,self).setUp()

# class Linear2D_CBISPFTMD_DerivativeChecks(Linear2D_TMD_TestCase,
#                                           CommonBasisIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Linear2D_CBISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Linear2D_CBISPFTMD_DerivativeChecks,self).setUp()

# class Banana2D_CBISPFTMD_DerivativeChecks(Banana2D_TMD_TestCase,
#                                           CommonBasisIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Banana2D_CBISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Banana2D_CBISPFTMD_DerivativeChecks,self).setUp()

# TOTAL ORDER INTEGRATED SQUARED PUSHFORWARD
class Linear1D_TOISPFTMD_DerivativeChecks(Linear1D_TMD_TestCase,
                                        TotOrdIntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Linear1D_TOISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear1D_TOISPFTMD_DerivativeChecks,self).setUp()

class ArcTan1D_TOISPFTMD_DerivativeChecks(ArcTan1D_TMD_TestCase,
                                        TotOrdIntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(ArcTan1D_TOISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(ArcTan1D_TOISPFTMD_DerivativeChecks,self).setUp()

class Exp1D_TOISPFTMD_DerivativeChecks(Exp1D_TMD_TestCase,
                                     TotOrdIntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Exp1D_TOISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Exp1D_TOISPFTMD_DerivativeChecks,self).setUp()

class Logistic1D_TOISPFTMD_DerivativeChecks(Logistic1D_TMD_TestCase,
                                          TotOrdIntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Logistic1D_TOISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Logistic1D_TOISPFTMD_DerivativeChecks,self).setUp()

class Gamma1D_TOISPFTMD_DerivativeChecks(Gamma1D_TMD_TestCase,
                                       TotOrdIntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Gamma1D_TOISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Gamma1D_TOISPFTMD_DerivativeChecks,self).setUp()

class Beta1D_TOISPFTMD_DerivativeChecks(Beta1D_TMD_TestCase,
                                      TotOrdIntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Beta1D_TOISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Beta1D_TOISPFTMD_DerivativeChecks,self).setUp()

class Gumbel1D_TOISPFTMD_DerivativeChecks(Gumbel1D_TMD_TestCase,
                                        TotOrdIntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Gumbel1D_TOISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Gumbel1D_TOISPFTMD_DerivativeChecks,self).setUp()

class Linear2D_TOISPFTMD_DerivativeChecks(Linear2D_TMD_TestCase,
                                        TotOrdIntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Linear2D_TOISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear2D_TOISPFTMD_DerivativeChecks,self).setUp()

class Banana2D_TOISPFTMD_DerivativeChecks(Banana2D_TMD_TestCase,
                                        TotOrdIntegratedSquaredPFTMD_DerivativeChecks):
    def setUp(self):
        super(Banana2D_TOISPFTMD_DerivativeChecks,self).setUp_test_case()
        super(Banana2D_TOISPFTMD_DerivativeChecks,self).setUp()

# # COMMON BASIS TOTAL ORDER INTEGRATED SQUARED PUSHFORWARD
# class Linear1D_CBTOISPFTMD_DerivativeChecks(
#         Linear1D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Linear1D_CBTOISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Linear1D_CBTOISPFTMD_DerivativeChecks,self).setUp()

# class ArcTan1D_CBTOISPFTMD_DerivativeChecks(
#         ArcTan1D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(ArcTan1D_CBTOISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(ArcTan1D_CBTOISPFTMD_DerivativeChecks,self).setUp()

# class Exp1D_CBTOISPFTMD_DerivativeChecks(
#         Exp1D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Exp1D_CBTOISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Exp1D_CBTOISPFTMD_DerivativeChecks,self).setUp()

# class Logistic1D_CBTOISPFTMD_DerivativeChecks(
#         Logistic1D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Logistic1D_CBTOISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Logistic1D_CBTOISPFTMD_DerivativeChecks,self).setUp()

# class Gamma1D_CBTOISPFTMD_DerivativeChecks(
#         Gamma1D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Gamma1D_CBTOISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Gamma1D_CBTOISPFTMD_DerivativeChecks,self).setUp()

# class Beta1D_CBTOISPFTMD_DerivativeChecks(
#         Beta1D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Beta1D_CBTOISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Beta1D_CBTOISPFTMD_DerivativeChecks,self).setUp()

# class Gumbel1D_CBTOISPFTMD_DerivativeChecks(
#         Gumbel1D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Gumbel1D_CBTOISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Gumbel1D_CBTOISPFTMD_DerivativeChecks,self).setUp()

# class Linear2D_CBTOISPFTMD_DerivativeChecks(
#         Linear2D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Linear2D_CBTOISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Linear2D_CBTOISPFTMD_DerivativeChecks,self).setUp()

# class Banana2D_CBTOISPFTMD_DerivativeChecks(
#         Banana2D_TMD_TestCase,
#         CommonBasisTotOrdIntegratedSquaredPFTMD_DerivativeChecks):
#     def setUp(self):
#         super(Banana2D_CBTOISPFTMD_DerivativeChecks,self).setUp_test_case()
#         super(Banana2D_CBTOISPFTMD_DerivativeChecks,self).setUp()

        
# LINEAR APPROXIMATIONS
class Linear1D_LSPBTMD_DerivativeChecks(Linear1D_TMD_TestCase,
                                        LinearSpanPBTMD_DerivativeChecks):

    def setUp(self):
        super(Linear1D_LSPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear1D_LSPBTMD_DerivativeChecks,self).setUp()

class Linear1D_CBLSPBTMD_DerivativeChecks(Linear1D_TMD_TestCase,
                                          CommonBasisLinearSpanPBTMD_DerivativeChecks):

    def setUp(self):
        super(Linear1D_CBLSPBTMD_DerivativeChecks,self).setUp_test_case()
        super(Linear1D_CBLSPBTMD_DerivativeChecks,self).setUp()

def build_suite(ttype='all'):
    # Integrated exponential pullback
    suite_linear1d_iepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear1D_IEPBTMD_DerivativeChecks )
    suite_arctan1d_iepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        ArcTan1D_IEPBTMD_DerivativeChecks )
    suite_exp1d_iepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Exp1D_IEPBTMD_DerivativeChecks )
    suite_logistic1d_iepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Logistic1D_IEPBTMD_DerivativeChecks )
    suite_gamma1d_iepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Gamma1D_IEPBTMD_DerivativeChecks )
    suite_beta1d_iepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Beta1D_IEPBTMD_DerivativeChecks )
    suite_gumbel1d_iepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Gumbel1D_IEPBTMD_DerivativeChecks )
    suite_linear2d_iepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear2D_IEPBTMD_DerivativeChecks )
    suite_banana2d_iepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Banana2D_IEPBTMD_DerivativeChecks )
    # CommonBasis integrated exponential pullback
    suite_linear1d_cbiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear1D_CBIEPBTMD_DerivativeChecks )
    suite_arctan1d_cbiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        ArcTan1D_CBIEPBTMD_DerivativeChecks )
    suite_exp1d_cbiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Exp1D_CBIEPBTMD_DerivativeChecks )
    suite_logistic1d_cbiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Logistic1D_CBIEPBTMD_DerivativeChecks )
    suite_gamma1d_cbiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Gamma1D_CBIEPBTMD_DerivativeChecks )
    suite_beta1d_cbiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Beta1D_CBIEPBTMD_DerivativeChecks )
    suite_gumbel1d_cbiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Gumbel1D_CBIEPBTMD_DerivativeChecks )
    suite_linear2d_cbiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear2D_CBIEPBTMD_DerivativeChecks )
    suite_banana2d_cbiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Banana2D_CBIEPBTMD_DerivativeChecks )
    # Total order integrated exponential pullback
    suite_linear1d_toiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear1D_TOIEPBTMD_DerivativeChecks )
    suite_arctan1d_toiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        ArcTan1D_TOIEPBTMD_DerivativeChecks )
    suite_exp1d_toiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Exp1D_TOIEPBTMD_DerivativeChecks )
    suite_logistic1d_toiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Logistic1D_TOIEPBTMD_DerivativeChecks )
    suite_gamma1d_toiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Gamma1D_TOIEPBTMD_DerivativeChecks )
    suite_beta1d_toiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Beta1D_TOIEPBTMD_DerivativeChecks )
    suite_gumbel1d_toiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Gumbel1D_TOIEPBTMD_DerivativeChecks )
    suite_linear2d_toiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear2D_TOIEPBTMD_DerivativeChecks )
    suite_banana2d_toiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Banana2D_TOIEPBTMD_DerivativeChecks )
    # Total order commonBasis integrated exponential pullback
    suite_linear1d_cbtoiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear1D_CBTOIEPBTMD_DerivativeChecks )
    suite_arctan1d_cbtoiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        ArcTan1D_CBTOIEPBTMD_DerivativeChecks )
    suite_exp1d_cbtoiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Exp1D_CBTOIEPBTMD_DerivativeChecks )
    suite_logistic1d_cbtoiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Logistic1D_CBTOIEPBTMD_DerivativeChecks )
    suite_gamma1d_cbtoiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Gamma1D_CBTOIEPBTMD_DerivativeChecks )
    suite_beta1d_cbtoiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Beta1D_CBTOIEPBTMD_DerivativeChecks )
    suite_gumbel1d_cbtoiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Gumbel1D_CBTOIEPBTMD_DerivativeChecks )
    suite_linear2d_cbtoiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear2D_CBTOIEPBTMD_DerivativeChecks )
    suite_banana2d_cbtoiepbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Banana2D_CBTOIEPBTMD_DerivativeChecks )
    # Integrated exponential pushforward
    suite_linear1d_iepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear1D_IEPFTMD_DerivativeChecks )
    suite_arctan1d_iepftmd = unittest.TestLoader().loadTestsFromTestCase(
        ArcTan1D_IEPFTMD_DerivativeChecks )
    suite_exp1d_iepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Exp1D_IEPFTMD_DerivativeChecks )
    suite_logistic1d_iepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Logistic1D_IEPFTMD_DerivativeChecks )
    suite_gamma1d_iepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Gamma1D_IEPFTMD_DerivativeChecks )
    suite_beta1d_iepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Beta1D_IEPFTMD_DerivativeChecks )
    suite_gumbel1d_iepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Gumbel1D_IEPFTMD_DerivativeChecks )
    suite_linear2d_iepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear2D_IEPFTMD_DerivativeChecks )
    suite_banana2d_iepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Banana2D_IEPFTMD_DerivativeChecks )
    # CommonBasis integrated exponential pushforward
    suite_linear1d_cbiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear1D_CBIEPFTMD_DerivativeChecks )
    suite_arctan1d_cbiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        ArcTan1D_CBIEPFTMD_DerivativeChecks )
    suite_exp1d_cbiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Exp1D_CBIEPFTMD_DerivativeChecks )
    suite_logistic1d_cbiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Logistic1D_CBIEPFTMD_DerivativeChecks )
    suite_gamma1d_cbiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Gamma1D_CBIEPFTMD_DerivativeChecks )
    suite_beta1d_cbiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Beta1D_CBIEPFTMD_DerivativeChecks )
    suite_gumbel1d_cbiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Gumbel1D_CBIEPFTMD_DerivativeChecks )
    suite_linear2d_cbiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear2D_CBIEPFTMD_DerivativeChecks )
    suite_banana2d_cbiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Banana2D_CBIEPFTMD_DerivativeChecks )
    # Total order integrated exponential pushforward
    suite_linear1d_toiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear1D_TOIEPFTMD_DerivativeChecks )
    suite_arctan1d_toiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        ArcTan1D_TOIEPFTMD_DerivativeChecks )
    suite_exp1d_toiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Exp1D_TOIEPFTMD_DerivativeChecks )
    suite_logistic1d_toiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Logistic1D_TOIEPFTMD_DerivativeChecks )
    suite_gamma1d_toiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Gamma1D_TOIEPFTMD_DerivativeChecks )
    suite_beta1d_toiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Beta1D_TOIEPFTMD_DerivativeChecks )
    suite_gumbel1d_toiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Gumbel1D_TOIEPFTMD_DerivativeChecks )
    suite_linear2d_toiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear2D_TOIEPFTMD_DerivativeChecks )
    suite_banana2d_toiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Banana2D_TOIEPFTMD_DerivativeChecks )
    # Total order commonBasis integrated exponential pushforward
    suite_linear1d_cbtoiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear1D_CBTOIEPFTMD_DerivativeChecks )
    suite_arctan1d_cbtoiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        ArcTan1D_CBTOIEPFTMD_DerivativeChecks )
    suite_exp1d_cbtoiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Exp1D_CBTOIEPFTMD_DerivativeChecks )
    suite_logistic1d_cbtoiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Logistic1D_CBTOIEPFTMD_DerivativeChecks )
    suite_gamma1d_cbtoiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Gamma1D_CBTOIEPFTMD_DerivativeChecks )
    suite_beta1d_cbtoiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Beta1D_CBTOIEPFTMD_DerivativeChecks )
    suite_gumbel1d_cbtoiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Gumbel1D_CBTOIEPFTMD_DerivativeChecks )
    suite_linear2d_cbtoiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear2D_CBTOIEPFTMD_DerivativeChecks )
    suite_banana2d_cbtoiepftmd = unittest.TestLoader().loadTestsFromTestCase(
        Banana2D_CBTOIEPFTMD_DerivativeChecks )
    # Integrated squared pullback
    suite_linear1d_ispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear1D_ISPBTMD_DerivativeChecks )
    suite_arctan1d_ispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        ArcTan1D_ISPBTMD_DerivativeChecks )
    suite_exp1d_ispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Exp1D_ISPBTMD_DerivativeChecks )
    suite_logistic1d_ispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Logistic1D_ISPBTMD_DerivativeChecks )
    suite_gamma1d_ispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Gamma1D_ISPBTMD_DerivativeChecks )
    suite_beta1d_ispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Beta1D_ISPBTMD_DerivativeChecks )
    suite_gumbel1d_ispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Gumbel1D_ISPBTMD_DerivativeChecks )
    suite_linear2d_ispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear2D_ISPBTMD_DerivativeChecks )
    suite_banana2d_ispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Banana2D_ISPBTMD_DerivativeChecks )
    # # Common basis integrated squared pullback
    # suite_linear1d_cbispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Linear1D_CBISPBTMD_DerivativeChecks )
    # suite_arctan1d_cbispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     ArcTan1D_CBISPBTMD_DerivativeChecks )
    # suite_exp1d_cbispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Exp1D_CBISPBTMD_DerivativeChecks )
    # suite_logistic1d_cbispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Logistic1D_CBISPBTMD_DerivativeChecks )
    # suite_gamma1d_cbispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Gamma1D_CBISPBTMD_DerivativeChecks )
    # suite_beta1d_cbispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Beta1D_CBISPBTMD_DerivativeChecks )
    # suite_gumbel1d_cbispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Gumbel1D_CBISPBTMD_DerivativeChecks )
    # suite_linear2d_cbispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Linear2D_CBISPBTMD_DerivativeChecks )
    # suite_banana2d_cbispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Banana2D_CBISPBTMD_DerivativeChecks )
    # Total order integrated squared pullback
    suite_linear1d_toispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear1D_TOISPBTMD_DerivativeChecks )
    suite_arctan1d_toispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        ArcTan1D_TOISPBTMD_DerivativeChecks )
    suite_exp1d_toispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Exp1D_TOISPBTMD_DerivativeChecks )
    suite_logistic1d_toispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Logistic1D_TOISPBTMD_DerivativeChecks )
    suite_gamma1d_toispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Gamma1D_TOISPBTMD_DerivativeChecks )
    suite_beta1d_toispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Beta1D_TOISPBTMD_DerivativeChecks )
    suite_gumbel1d_toispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Gumbel1D_TOISPBTMD_DerivativeChecks )
    suite_linear2d_toispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear2D_TOISPBTMD_DerivativeChecks )
    suite_banana2d_toispbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Banana2D_TOISPBTMD_DerivativeChecks )
    # # Total order commonBasis integrated squared pullback
    # suite_linear1d_cbtoispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Linear1D_CBTOISPBTMD_DerivativeChecks )
    # suite_arctan1d_cbtoispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     ArcTan1D_CBTOISPBTMD_DerivativeChecks )
    # suite_exp1d_cbtoispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Exp1D_CBTOISPBTMD_DerivativeChecks )
    # suite_logistic1d_cbtoispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Logistic1D_CBTOISPBTMD_DerivativeChecks )
    # suite_gamma1d_cbtoispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Gamma1D_CBTOISPBTMD_DerivativeChecks )
    # suite_beta1d_cbtoispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Beta1D_CBTOISPBTMD_DerivativeChecks )
    # suite_gumbel1d_cbtoispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Gumbel1D_CBTOISPBTMD_DerivativeChecks )
    # suite_linear2d_cbtoispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Linear2D_CBTOISPBTMD_DerivativeChecks )
    # suite_banana2d_cbtoispbtmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Banana2D_CBTOISPBTMD_DerivativeChecks )
    # Integrated squared pushforward
    suite_linear1d_ispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear1D_ISPFTMD_DerivativeChecks )
    suite_arctan1d_ispftmd = unittest.TestLoader().loadTestsFromTestCase(
        ArcTan1D_ISPFTMD_DerivativeChecks )
    suite_exp1d_ispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Exp1D_ISPFTMD_DerivativeChecks )
    suite_logistic1d_ispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Logistic1D_ISPFTMD_DerivativeChecks )
    suite_gamma1d_ispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Gamma1D_ISPFTMD_DerivativeChecks )
    suite_beta1d_ispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Beta1D_ISPFTMD_DerivativeChecks )
    suite_gumbel1d_ispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Gumbel1D_ISPFTMD_DerivativeChecks )
    suite_linear2d_ispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear2D_ISPFTMD_DerivativeChecks )
    suite_banana2d_ispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Banana2D_ISPFTMD_DerivativeChecks )
    # # CommonBasis integrated squared pushforward
    # suite_linear1d_cbispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Linear1D_CBISPFTMD_DerivativeChecks )
    # suite_arctan1d_cbispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     ArcTan1D_CBISPFTMD_DerivativeChecks )
    # suite_exp1d_cbispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Exp1D_CBISPFTMD_DerivativeChecks )
    # suite_logistic1d_cbispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Logistic1D_CBISPFTMD_DerivativeChecks )
    # suite_gamma1d_cbispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Gamma1D_CBISPFTMD_DerivativeChecks )
    # suite_beta1d_cbispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Beta1D_CBISPFTMD_DerivativeChecks )
    # suite_gumbel1d_cbispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Gumbel1D_CBISPFTMD_DerivativeChecks )
    # suite_linear2d_cbispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Linear2D_CBISPFTMD_DerivativeChecks )
    # suite_banana2d_cbispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Banana2D_CBISPFTMD_DerivativeChecks )
    # Total order integrated squared pushforward
    suite_linear1d_toispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear1D_TOISPFTMD_DerivativeChecks )
    suite_arctan1d_toispftmd = unittest.TestLoader().loadTestsFromTestCase(
        ArcTan1D_TOISPFTMD_DerivativeChecks )
    suite_exp1d_toispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Exp1D_TOISPFTMD_DerivativeChecks )
    suite_logistic1d_toispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Logistic1D_TOISPFTMD_DerivativeChecks )
    suite_gamma1d_toispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Gamma1D_TOISPFTMD_DerivativeChecks )
    suite_beta1d_toispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Beta1D_TOISPFTMD_DerivativeChecks )
    suite_gumbel1d_toispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Gumbel1D_TOISPFTMD_DerivativeChecks )
    suite_linear2d_toispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear2D_TOISPFTMD_DerivativeChecks )
    suite_banana2d_toispftmd = unittest.TestLoader().loadTestsFromTestCase(
        Banana2D_TOISPFTMD_DerivativeChecks )
    # # Total order commonBasis integrated squared pushforward
    # suite_linear1d_cbtoispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Linear1D_CBTOISPFTMD_DerivativeChecks )
    # suite_arctan1d_cbtoispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     ArcTan1D_CBTOISPFTMD_DerivativeChecks )
    # suite_exp1d_cbtoispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Exp1D_CBTOISPFTMD_DerivativeChecks )
    # suite_logistic1d_cbtoispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Logistic1D_CBTOISPFTMD_DerivativeChecks )
    # suite_gamma1d_cbtoispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Gamma1D_CBTOISPFTMD_DerivativeChecks )
    # suite_beta1d_cbtoispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Beta1D_CBTOISPFTMD_DerivativeChecks )
    # suite_gumbel1d_cbtoispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Gumbel1D_CBTOISPFTMD_DerivativeChecks )
    # suite_linear2d_cbtoispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Linear2D_CBTOISPFTMD_DerivativeChecks )
    # suite_banana2d_cbtoispftmd = unittest.TestLoader().loadTestsFromTestCase(
    #     Banana2D_CBTOISPFTMD_DerivativeChecks )
    
    # Linear Span
    suite_linear1d_lspbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear1D_LSPBTMD_DerivativeChecks )
    suite_linear1d_cblspbtmd = unittest.TestLoader().loadTestsFromTestCase(
        Linear1D_CBLSPBTMD_DerivativeChecks )

    # GROUP SUITES
    suites_list = []
    if ttype in ['all','serial']:
        suites_list += [
            # Integrated exponential pullback
            suite_linear1d_iepbtmd, suite_arctan1d_iepbtmd, suite_exp1d_iepbtmd,
            suite_logistic1d_iepbtmd, suite_gamma1d_iepbtmd, suite_beta1d_iepbtmd,
            suite_gumbel1d_iepbtmd, suite_linear2d_iepbtmd,
            suite_banana2d_iepbtmd,
            # Common basis integrated exponential pullback
            suite_linear1d_cbiepbtmd, suite_arctan1d_cbiepbtmd, suite_exp1d_cbiepbtmd,
            suite_logistic1d_cbiepbtmd, suite_gamma1d_cbiepbtmd, suite_beta1d_cbiepbtmd,
            suite_gumbel1d_cbiepbtmd, suite_linear2d_cbiepbtmd, suite_banana2d_cbiepbtmd,
            # Total order integrated exponential pullback
            suite_linear1d_toiepbtmd, suite_arctan1d_toiepbtmd, suite_exp1d_toiepbtmd,
            suite_logistic1d_toiepbtmd, suite_gamma1d_toiepbtmd, suite_beta1d_toiepbtmd,
            suite_gumbel1d_toiepbtmd, suite_linear2d_toiepbtmd, suite_banana2d_toiepbtmd,
            # Common basis total order integrated exponential pullback
            suite_linear1d_cbtoiepbtmd, suite_arctan1d_cbtoiepbtmd, suite_exp1d_cbtoiepbtmd,
            suite_logistic1d_cbtoiepbtmd, suite_gamma1d_cbtoiepbtmd,
            suite_beta1d_cbtoiepbtmd,
            suite_gumbel1d_cbtoiepbtmd, suite_linear2d_cbtoiepbtmd,
            suite_banana2d_cbtoiepbtmd,
            # Integrated exponential pushforward
            suite_linear1d_iepftmd, suite_arctan1d_iepftmd, suite_exp1d_iepftmd,
            suite_logistic1d_iepftmd, suite_gamma1d_iepftmd, suite_beta1d_iepftmd,
            suite_gumbel1d_iepftmd, suite_linear2d_iepftmd, suite_banana2d_iepftmd,
            # Common basis integrated exponential pushforward
            suite_linear1d_cbiepftmd, suite_arctan1d_cbiepftmd, suite_exp1d_cbiepftmd,
            suite_logistic1d_cbiepftmd, suite_gamma1d_cbiepftmd, suite_beta1d_cbiepftmd,
            suite_gumbel1d_cbiepftmd, suite_linear2d_cbiepftmd, suite_banana2d_cbiepftmd,
            # Total order integrated exponential pushforward
            suite_linear1d_toiepftmd, suite_arctan1d_toiepftmd, suite_exp1d_toiepftmd,
            suite_logistic1d_toiepftmd, suite_gamma1d_toiepftmd, suite_beta1d_toiepftmd,
            suite_gumbel1d_toiepftmd, suite_linear2d_toiepftmd, suite_banana2d_toiepftmd,
            # Common basis total order integrated exponential pushforward
            suite_linear1d_cbtoiepftmd, suite_arctan1d_cbtoiepftmd, suite_exp1d_cbtoiepftmd,
            suite_logistic1d_cbtoiepftmd, suite_gamma1d_cbtoiepftmd,
            suite_beta1d_cbtoiepftmd,
            suite_gumbel1d_cbtoiepftmd, suite_linear2d_cbtoiepftmd,
            suite_banana2d_cbtoiepftmd,

            # INTEGRATED SQUARED PULLBACK
            suite_linear1d_ispbtmd, suite_arctan1d_ispbtmd, suite_exp1d_ispbtmd,
            suite_logistic1d_ispbtmd, suite_gamma1d_ispbtmd, suite_beta1d_ispbtmd,
            suite_gumbel1d_ispbtmd, suite_linear2d_ispbtmd,
            suite_banana2d_ispbtmd,
            # # COMMON BASIS INTEGRATED SQUARED PULLBACK
            # suite_linear1d_cbispbtmd, suite_arctan1d_cbispbtmd, suite_exp1d_cbispbtmd,
            # suite_logistic1d_cbispbtmd, suite_gamma1d_cbispbtmd, suite_beta1d_cbispbtmd,
            # suite_gumbel1d_cbispbtmd, suite_linear2d_cbispbtmd, suite_banana2d_cbispbtmd,
            # TOTAL ORDER INTEGRATED SQUARED PULLBACK
            suite_linear1d_toispbtmd, suite_arctan1d_toispbtmd, suite_exp1d_toispbtmd,
            suite_logistic1d_toispbtmd, suite_gamma1d_toispbtmd, suite_beta1d_toispbtmd,
            suite_gumbel1d_toispbtmd, suite_linear2d_toispbtmd, suite_banana2d_toispbtmd,
            # # COMMON BASIS TOTAL ORDER INTEGRATED SQUARED PULLBACK
            # suite_linear1d_cbtoispbtmd, suite_arctan1d_cbtoispbtmd, suite_exp1d_cbtoispbtmd,
            # suite_logistic1d_cbtoispbtmd, suite_gamma1d_cbtoispbtmd,
            # suite_beta1d_cbtoispbtmd,
            # suite_gumbel1d_cbtoispbtmd, suite_linear2d_cbtoispbtmd,
            # suite_banana2d_cbtoispbtmd,
            # INTEGRATED SQUARED PUSHFORWARD
            suite_linear1d_ispftmd, suite_arctan1d_ispftmd, suite_exp1d_ispftmd,
            suite_logistic1d_ispftmd, suite_gamma1d_ispftmd, suite_beta1d_ispftmd,
            suite_gumbel1d_ispftmd, suite_linear2d_ispftmd, suite_banana2d_ispftmd,
            # # COMMON BASIS INTEGRATED SQUARED PUSHFORWARD
            # suite_linear1d_cbispftmd, suite_arctan1d_cbispftmd, suite_exp1d_cbispftmd,
            # suite_logistic1d_cbispftmd, suite_gamma1d_cbispftmd, suite_beta1d_cbispftmd,
            # suite_gumbel1d_cbispftmd, suite_linear2d_cbispftmd, suite_banana2d_cbispftmd,
            # TOTAL ORDER INTEGRATED SQUARED PUSHFORWARD
            suite_linear1d_toispftmd, suite_arctan1d_toispftmd, suite_exp1d_toispftmd,
            suite_logistic1d_toispftmd, suite_gamma1d_toispftmd, suite_beta1d_toispftmd,
            suite_gumbel1d_toispftmd, suite_linear2d_toispftmd, suite_banana2d_toispftmd,
            # # COMMON BASIS TOTAL ORDER INTEGRATED SQUARED PUSHFORWARD
            # suite_linear1d_cbtoispftmd, suite_arctan1d_cbtoispftmd, suite_exp1d_cbtoispftmd,
            # suite_logistic1d_cbtoispftmd, suite_gamma1d_cbtoispftmd,
            # suite_beta1d_cbtoispftmd,
            # suite_gumbel1d_cbtoispftmd, suite_linear2d_cbtoispftmd,
            # suite_banana2d_cbtoispftmd

            # LINEAR SPAN PULLBACK
            suite_linear1d_lspbtmd, suite_linear1d_cblspbtmd,
        ]

    all_suites = unittest.TestSuite( suites_list )
    return all_suites

def run_tests(ttype='all'):
    all_suites = build_suite(ttype)
    # RUN
    unittest.TextTestRunner(verbosity=2).run(all_suites)

if __name__ == '__main__':
    run_tests()
